#! /usr/bin/env python

from distutils.core import setup
from distutils.extension import Extension

try:
    from Cython.Distutils import build_ext
    from Cython.Compiler import Main
    has_cython=True
except ImportError:
    from distutils.command.build_ext import build_ext
    has_cython=False

import re, sys
import glob

from os import path

def findPackage(root,base=None):
    modules=[]
    if base is None:
        base=[]
    for module in (path.basename(path.dirname(x)) 
                   for x in glob.glob(path.join(root,'*','__init__.py'))):
        modules.append('.'.join(base+[module]))
        modules.extend(findPackage(path.join(root,module),base+[module]))
    return modules
    
def findCython(root,base=None,pyrexs=None):
    modules=[]
    pyrexs=[]
    #o=dict(Main.default_options)
    pyopt=Main.CompilationOptions(Main.default_options)
    Main.__dict__['context'] = Main.Context(pyopt.include_path, pyopt.pragma_overrides)
    if base is None:
        base=[]
    for module in (path.basename(path.dirname(x)) 
                   for x in glob.glob(path.join(root,'*','__init__.py'))):
                
        for pyrex in glob.glob(path.join(root,module,'*.pyx')):
            pyrexs.append(Extension('.'.join(base+[module,path.splitext(path.basename(pyrex))[0]]),[pyrex]))
            Main.compile([pyrex],timestamps=True,recursion=True)
            
        pyrexs.extend(findCython(path.join(root,module),base+[module]))
    return pyrexs
    
def findC(root,base=None,pyrexs=None):
    modules=[]
    pyrexs=[]
    if base is None:
        base=[]
    for module in (path.basename(path.dirname(x)) 
                   for x in glob.glob(path.join(root,'*','__init__.py'))):
                
        for pyrex in glob.glob(path.join(root,module,'*.c')):
            pyrexs.append(Extension('.'.join(base+[module,path.splitext(path.basename(pyrex))[0]]),[pyrex]))
        
        pyrexs.extend(findC(path.join(root,module),base+[module]))
    return pyrexs
    

VERSION = '0.1.3'
AUTHOR  = 'Eric Coissac'
EMAIL   = 'eric@coissac.eu'
URL     = 'www.grenoble.prabi.fr/trac/OBITools'
LICENSE = 'CeCILL-V2'

SRC = 'src'

SCRIPTS = glob.glob('%s/*.py' % SRC)

if has_cython:
    EXTENTION=findCython(SRC)
else:
    EXTENTION=findC(SRC)
    
#SCRIPTS.append('src/fastaComplement')

setup(name="OBITools",
      description="Scripts and library for sequence analysis",
      version=VERSION,
      author=AUTHOR,
      author_email=EMAIL,
      license=LICENSE,
      url=URL,
      scripts=SCRIPTS,
      package_dir = {'': SRC},
      packages=findPackage(SRC),
      cmdclass = {'build_ext': build_ext},
      ext_modules=EXTENTION)

