#!/usr/local/bin/python

import re

def igrep(lines,pattern,exclude=False):
    """
    igrep emule la commande unix grep.
    
        @param lines: une collection de string
        @type  lines: iterator on str
        @param pattern: une expression reguliere
        @type  pattern: str
        
        @return: un iterateur sur le sous ensemble 
                 des lignes possedant au moins une occurence
                 du pattern
                 
        @rtype: iterator on str
    """
    automate = re.compile(pattern)
    for l in lines:
        match = automate.search(l) is not None
        out = (match and not exclude) or (not match and exclude)
        if out:
            yield l

def wordIterator(sequence,lword,step=1,endIncluded=False):
    L = len(sequence)
    
    if endIncluded:
       pmax=L
    else:
       pmax = L - lword + 1
    
    pos = xrange(0,pmax,step)
    
    for x in pos:
        yield sequence[x:x+lword]
    


def fastaFormat(title,sequence):
    width = 50
    fasta = '>' + title + '\n'
    iwords= wordIterator(sequence,width,width,True)
    words =[w for w in iwords]
    seq = '\n'.join(words)
    return fasta + seq
    
def isqlines(lines):
    for l in igrep(lines,'^ '):
       yield l

def icleanSqLines(lines):
    sq = isqlines(lines)
    bad = re.compile('[ \t\n0-9]+')
    
    for line in sq:
        yield bad.sub('',line)
    
def getSeq(lines):
    sq = icleanSqLines(lines)
    allsq = [x for x in sq]
    seq = ''.join(allsq)
    return seq
    
    

if __name__=='__main__':
    
    # ici commence le main de mon script
    
    import sys
    import optparse
    
    parser = optparse.OptionParser()
    
    parser.add_option("-b", "--begin",
                      type="int", dest="begin",
                      default=1,
                      help="start position of the cutted fragment")

    parser.add_option("-e", "--end",
                      type="int", dest="end",
                      help="end position of the cutted fragment")

    parser.add_option("-t", "--title",
                      dest="title",
                      help="title line of the fasta output")

    (options, args) = parser.parse_args()
    
    
    if len(args) > 0:
        filename = args[0]
        f = open(filename)
    else:
        f = sys.stdin
        
    sq = getSeq(f)
    
    start = options.begin - 1
    stop  = options.end 
    
    subseq= sq[start:stop]
    
    fasta = fastaFormat(options.title,subseq)
    
    print fasta
        
