'''
Created on 13 oct. 2009

@author: coissac
'''

from obitools.format.sequence.embl import emblIterator
from obitools.format.sequence.genbank import genbankIterator
from obitools.format.sequence.fnaqual import fnaFastaIterator
from obitools.format.sequence.fasta import fastaAAIterator,fastaNucIterator,fastaIterator


def addInputFormatOption(optionManager):
    optionManager.add_option('--genbank',
                             action="store_const", dest="seqinformat",
                             default='fasta',
                             const='genbank',
                             help="input file is in genbank format")
    optionManager.add_option('--embl',
                             action="store_const", dest="seqinformat",
                             default='fasta',
                             const='embl',
                             help="input file is in embl format")

    optionManager.add_option('--fna',
                             action="store_const", dest="seqinformat",
                             default='fasta',
                             const='fna',
                             help="input file is in fasta nucleic format produced by 454 sequencer pipeline")

    optionManager.add_option('--nuc',
                             action="store_const", dest="moltype",
                             default=None,
                             const='nuc',
                             help="input file is nucleic sequences")
    optionManager.add_option('--prot',
                             action="store_const", dest="moltype",
                             default=None,
                             const='pep',
                             help="input file is protein sequences")

def entriesIterator(options):
    if options.seqinformat=='fasta':
        if options.moltype=='nuc':
            reader=fastaNucIterator
        elif options.moltype=='pep':
            reader=fastaAAIterator
        else:
            reader=fastaIterator
    elif options.seqinformat=='genbank':
        reader=genbankIterator
    elif options.seqinformat=='embl':
        reader=emblIterator
    elif options.seqinformat=='fna':
        reader=fnaFastaIterator
        
    def iterator(lineiterator):
        for s in reader(lineiterator):
            s.extractTaxon()
            yield s
            
    return iterator

        
    
    